@@ -1,4 +1,4 @@
-/* $OpenBSD: ts_lib.c,v 1.8 2014/07/11 08:44:49 jsing Exp $ */
+/* $OpenBSD: ts_lib.c,v 1.10 2015/09/10 14:29:22 jsing Exp $ */
 /* Written by Zoltan Glozik (zglozik@stones.com) for the OpenSSL
  * project 2002.
  */
@@ -137,14 +137,14 @@ TS_X509_ALGOR_print_bio(BIO *bio, const X509_ALGOR *alg)
 int
 TS_MSG_IMPRINT_print_bio(BIO *bio, TS_MSG_IMPRINT *a)
 {
-	const ASN1_OCTET_STRING *msg;
+	ASN1_OCTET_STRING *msg;
 
 	TS_X509_ALGOR_print_bio(bio, TS_MSG_IMPRINT_get_algo(a));
 
 	BIO_printf(bio, "Message data:\n");
 	msg = TS_MSG_IMPRINT_get_msg(a);
-	BIO_dump_indent(bio, (const char *)M_ASN1_STRING_data(msg),
-	    M_ASN1_STRING_length(msg), 4);
+	BIO_dump_indent(bio, (const char *)ASN1_STRING_data(msg),
+	    ASN1_STRING_length(msg), 4);
 
 	return 1;
 }
